#include <Python.h>

#include "cbarlib.h"
#include "cbarprivate.h"

#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qlayout.h>

#include <kfiledialog.h>
#include <kdebug.h>

static CbarWidget* cbarWidgetInstance = NULL;

static PyObject* pycbar_getCbarName(PyObject */*self*/, PyObject *args)
{
    if(!PyArg_ParseTuple(args, (char *)""))
        return NULL;
    return Py_BuildValue((char *)"s", cbarWidgetInstance->getCbarName().data());
}

typedef unsigned long long PtrConversionType;

static PyObject *pycbar_addCbarWidget(PyObject */*self*/, PyObject *args)
{
    kdDebug() << "Entering CbarBinding::addCbarWidget()\n";
    PtrConversionType ptrAsInt;
    PyObject *dictObject;
    if(!PyArg_ParseTuple(args, (char *)"KO", &ptrAsInt, &dictObject))
    {
        return NULL;
    }
    
    QWidget *widget = reinterpret_cast<QWidget*>(ptrAsInt);
    
    if (!PyDict_Check(dictObject))
    {
        return NULL;
    }
    
    PyObject *key;
    PyObject *value;
    int pos = 0;
    
    //Well, pyQt apparently does not support QMap and QList in signals :/
    QStringList params;
    
    while (PyDict_Next(dictObject, &pos, &key, &value))
    {
        QString keyString(PyString_AsString(key));
        QString valueString(PyString_AsString(value));
        params.push_back(keyString + "=" + valueString);
    }
    
    QObject *connectionObject = cbarWidgetInstance->addCbarWidget(
        widget,
        params
        );
    
    return Py_BuildValue((char *)"K", (PtrConversionType)connectionObject);
}

static PyObject *pycbar_executeScript(PyObject */*self*/, PyObject *args)
{
    kdDebug() << "Entering CbarBinding::executeScript()\n";
    
    const char *cstr;
    
    if(!PyArg_ParseTuple(args, (char *)"s", &cstr))
    {
        return NULL;
    }
    
    QString scriptPath(cstr);
    
    QTime startTime = QTime::currentTime();
    startTime.start();
    kdDebug() << "Starting running script: " << scriptPath << ", start time=" << startTime.toString("hh:mm:ss.zzz")  << " \n";
    
    bool result = cbarWidgetInstance->executeScript(
        scriptPath
        );
    
    QTime endTime = QTime::currentTime();
    kdDebug() << "Finished running script: " << scriptPath 
        << ", start time=" << startTime.toString("hh:mm:ss.zzz")
        << ", endTime=" << endTime.toString("hh:mm:ss.zzz")
        << ", duration=" <<  startTime.elapsed() << " ms"
        <<  " \n";
    
    return Py_BuildValue((char *)"i", (int)result);
}

static PyObject *pycbar_registerThemeManager(PyObject */*self*/, PyObject *args)
{
    kdDebug() << "Entering CbarBinding::registerThemeManager()\n";
    
    PtrConversionType ptrAsInt;
    if(!PyArg_ParseTuple(args, (char *)"K", &ptrAsInt))
    {
        return NULL;
    }
    QWidget *widget = reinterpret_cast<QWidget*>(ptrAsInt);
    QObject *connectionObject = cbarWidgetInstance->registerThemeManager(widget);
    
    return Py_BuildValue((char *)"K", (PtrConversionType)connectionObject);
}

static PyObject *pycbar_registerScriptLoader(PyObject */*self*/, PyObject *args)
{
    kdDebug() << "Entering CbarBinding::registerScriptLoader()\n";
    
    PtrConversionType ptrAsInt;
    if(!PyArg_ParseTuple(args, (char *)"K", &ptrAsInt))
    {
        return NULL;
    }
    QObject *object = reinterpret_cast<QObject*>(ptrAsInt);
    QObject *connectionObject = cbarWidgetInstance->registerScriptLoader(object);
    return Py_BuildValue((char *)"K", (PtrConversionType)connectionObject);
}

static PyMethodDef pythonMethods[] = 
{
    {(char *)"getCbarName", pycbar_getCbarName, METH_VARARGS, (char *)"Return name of Cbar instance."},
    {(char *)"addCbarWidget", pycbar_addCbarWidget, METH_VARARGS, (char *)"Adds widget to Cbar instance."},
    {(char *)"executeScript", pycbar_executeScript, METH_VARARGS, (char *)"Executes script in Cbar instance."},
    {(char *)"registerThemeManager", pycbar_registerThemeManager, METH_VARARGS, (char *)"Registers theme manager in Cbar instance."},
    {(char *)"registerScriptLoader", pycbar_registerScriptLoader, METH_VARARGS, (char *)"Registers script loader in Cbar instance."},
    {NULL, NULL, 0, NULL}
};


PyMODINIT_FUNC initcbarlib()
{
    Py_InitModule((char *)"cbarlib", pythonMethods);
}

void initializePython()
{
/*    Py_SetProgramName((char *)"cbar");*/
    if (! Py_IsInitialized())
    {
        Py_Initialize();
    }
    initcbarlib();
}

/************************* CbarBinding *********************/
CbarBinding::CbarBinding(
    CbarWidget *widget
    )
{
   cbarWidgetInstance = widget;
    initializePython();
}

CbarBinding::~CbarBinding() 
{
}

/************************* CbarWidget *********************/

QObject *CbarWidget::addCbarWidget(
    QWidget *newWidget,
    QStringList const & params
    )
{
/*    kdDebug() << "Emitting params, params=" << params << "\n";*/
    emit signalAddCbarWidget(newWidget, params);
    return this;
}

QObject *CbarWidget::registerThemeManager(QWidget *themeManager)
{
    themeManager->reparent(this, 0/*WFlags*/, QPoint(0, 0), true /*showIt*/);
    this->layout()->add(themeManager);
    connect(
        themeManager,
        SIGNAL(destroyed()),
        this->d.data(),
        SLOT(themeManagerDestroyed())
        );
    return this;
}

QObject *CbarWidget::registerScriptLoader(QObject *scriptLoaderObject)
{
    this->d->scriptLoader = scriptLoaderObject;
    return this;
}

QString CbarWidget::getCbarName()
{
    return this->d->cbarName;
}

bool CbarWidget::executeScript(
    QString const & scriptPath
    )
{
    return this->d->executeScript(scriptPath);
}


CbarWidget::CbarWidget(
    QString const & cbarName,
    QWidget * parent, 
    const char * name, 
    WFlags flags
    ):
    QWidget(
        parent,
        name,
        flags
        ),
    d(
        new CbarWidgetPrivate(
            this,
            cbarName
            )
        )
{
    this->setPaletteBackgroundColor(Qt::darkBlue);
    this->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
    new QVBoxLayout(this);

    this->d->loadScriptLoader();
}

void CbarWidget::openScriptManager()
{
    if (this->d->scriptLoader != NULL)
    {
        emit signalOpenScriptManager();
        return;
    }
    //fallback in case script manager is not registered
    this->openDefaultScriptManager();
}

void CbarWidget::openDefaultScriptManager()
{
    KFileDialog* filedialog = new KFileDialog(
        QString(),
        "*.py",
        NULL,
        "filedialog",
        FALSE //modal
        );
    
    filedialog->setCaption("Select script to run");
    if (!filedialog->exec())
    {
        delete filedialog;
        return;
    }
    
    QString filename = filedialog->selectedFile();
    delete filedialog;

    this->d->executeScript(filename);
}

void CbarWidget::itemsSelected(const KFileItemList & items)
{
    emit signalItemsSelected(items);
}

void CbarWidget::urlOpened(const QString & url)
{
    emit signalUrlOpened(url);
}

/************************* CbarFactory *********************/

CbarWidget::Ptr CbarFactory::createCbarWidget(
    QString const & cbarName,
    QWidget *parent
    )
{
    return CbarWidget::Ptr(
        new CbarWidget(
            cbarName, 
            parent
            )
        );
}

#include "cbarlib.moc"
