#ifndef __CBARLIB_H__
#define __CBARLIB_H__

#include <qvbox.h>
#include <ksharedptr.h>
#include <kfileitem.h>

class CbarWidgetPrivate;

class CbarWidget:
    public QWidget,
    public KShared
{
    Q_OBJECT;
public:
    typedef KSharedPtr<CbarWidget> Ptr;
    
public slots:
    /**
        Opens Cbar script manager.
    */
    void openScriptManager();
    
    /**
        Opens default (basic) script manager, implemented in C++.
    */
    void openDefaultScriptManager();
    
    /**
        Items has been selected in attached view, call scripts to handle it.
    */
    void itemsSelected(const KFileItemList & items);
    
    /**
        URL has been opened in view, for example directory has been entered.
    */
    void urlOpened(const QString & url);
    
    /**
        Adds given widget to display in Cbar.
        @param newWidget Pointer to widget which will be added.
        @param params Key, value pairs, describing properties of widget and how it should be handled.
        @returns Pointer to object which will be used as connection point for slots.
        
        Widget will be reparented to this widget.
    */
    QObject *addCbarWidget(
        QWidget *newWidget,
        QStringList const & params
        );
    
    /**
        Registers theme manager in Cbar.
        
        @returns Pointer to object which will be used as connection point for slots.
    */
    QObject *registerThemeManager(QWidget *newWidget);
    
    /**
        Registers script loader in Cbar.
        
        @returns Pointer to object which will be used as connection point for slots.
    */
    QObject *registerScriptLoader(QObject *object);
    
    /**
        Returns this Cbar name.
    */
    QString getCbarName();
    
    /**
        Executes script.
        @param scriptPath Path to script which should be executed.
    */
    bool executeScript(
        QString const & scriptPath
        );
    
signals:
    void signalItemsSelected(const KFileItemList & items);
    void signalUrlOpened(const QString & url);
    void signalAddCbarWidget(
        QWidget *widget,
        QStringList params
        );
    void signalOpenScriptManager();
    
protected:
    /**
        Constructor, for use in subclasses only. 
        
        To create instance for usage in apps, use CbarLib::createCbarWidget.
    */
    CbarWidget(
        QString const & cbarName,
        QWidget * parent = 0, 
        const char * name = 0, 
        WFlags flags = 0
        );
    
    KSharedPtr<CbarWidgetPrivate> d;
    
    friend class CbarFactory; //to use constructor
};

class CbarFactory
{
public:
    /**
        Creates instance of CbarWidget.
    */
    static CbarWidget::Ptr createCbarWidget(
        QString const & cbarName,
        QWidget *parent
        );
};

#endif //#ifndef __CBARLIB_H__
